ul.g-admin-options
  list-style-type none
  font-size 18px
  padding-left 0
  margin 0

  >li
    display block
    padding 0
    font-weight bold
    color #666

    >a
      padding 12px 10px
      color inherit
      display block

  >li:hover
    background-color #f4f4f4
    cursor pointer
    color #222
